#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "./vec.h"

#define SUCCESS 0
#define ERROR_NO_FURTHER_INPUT -1
#define ERROR_STRING_IS_NOT_A_NUMBER -2

int read_int(int *user_input) {
  char *line = NULL;
  size_t line_len = 0;
  ssize_t status = getline(&line, &line_len, stdin);
  if (status < 0) {
    free(line);
    return ERROR_NO_FURTHER_INPUT;
  }
  errno = 0;
  char *end = NULL;
  int i = strtol(line, &end, 10);
  free(line);
  if (errno != 0 || end == line) {
    return ERROR_STRING_IS_NOT_A_NUMBER;
  } else {
    *user_input = i;
    return SUCCESS;
  }
}

int main(int argc, char **argv) {
  Vec *data_points = vec_new();
  size_t num_invalid_data_points = 0;
  while (1) {
    int user_input;
    int status = read_int(&user_input);
    if (status == ERROR_NO_FURTHER_INPUT) {
      break;
    } else if (status == ERROR_STRING_IS_NOT_A_NUMBER) {
      num_invalid_data_points++;
    } else {
      vec_push(data_points, user_input);
    }
  }

  vec_sort(data_points);

  size_t len = vec_length(data_points);
  int minimum = *vec_at(data_points, 0 * len / 4);
  int lower_quartile = *vec_at(data_points, 1 * len / 4);
  int median = *vec_at(data_points, 2 * len / 4);
  int upper_quartile = *vec_at(data_points, 3 * len / 4);
  int maximum = *vec_at(data_points, len - 1);

  int sum = 0;
  for (size_t i = 0; i < len; ++i) {
    sum += *vec_at(data_points, i);
  }
  float average = (float)sum / (float)len;

  printf("Number of data points: %ld\n", len);
  printf("Minimum:               %d\n", minimum);
  printf("Lower Quartile:        %d\n", lower_quartile);
  printf("Median:                %d\n", median);
  printf("Upper Quartile:        %d\n", upper_quartile);
  printf("Maximum:               %d\n", maximum);
  printf("Average:               %f\n", average);
  if (num_invalid_data_points > 0) {
    printf("\nWARNING: Found %ld invalid data points.\n",
           num_invalid_data_points);
  }

  vec_free(data_points);
}
