#include "../unity/unity.h"
#include "./vec.h"

void setUp(void) {}
void tearDown(void) {}

void test_alloc(void) {
  Vec *xs = vec_new();

  TEST_ASSERT_EQUAL_INT(0, vec_length(xs));
  TEST_ASSERT_EQUAL_INT(1, vec_capacity(xs));

  vec_free(xs);
}

void test_push(void) {
  Vec *xs = vec_new();

  vec_push(xs, 10);
  vec_push(xs, 20);
  vec_push(xs, 30);
  vec_push(xs, 40);
  vec_push(xs, 50);

  TEST_ASSERT_EQUAL_INT(5, vec_length(xs));
  TEST_ASSERT_EQUAL_INT(10, *vec_at(xs, 0));
  TEST_ASSERT_EQUAL_INT(20, *vec_at(xs, 1));
  TEST_ASSERT_EQUAL_INT(30, *vec_at(xs, 2));
  TEST_ASSERT_EQUAL_INT(40, *vec_at(xs, 3));
  TEST_ASSERT_EQUAL_INT(50, *vec_at(xs, 4));

  vec_free(xs);
}

void test_pop(void) {
  Vec *xs = vec_new();

  vec_push(xs, 10);
  vec_push(xs, 20);
  vec_push(xs, 30);
  vec_pop(xs);

  TEST_ASSERT_EQUAL_INT(2, vec_length(xs));
  TEST_ASSERT_EQUAL_INT(10, *vec_at(xs, 0));
  TEST_ASSERT_EQUAL_INT(20, *vec_at(xs, 1));

  vec_free(xs);
}

void test_min_between(void) {
  Vec *xs = vec_new();

  vec_push(xs, 10);
  vec_push(xs, 20);
  vec_push(xs, 40);
  vec_push(xs, 30);
  vec_push(xs, 50);
  vec_push(xs, 60);

  TEST_ASSERT_EQUAL_INT(30, *vec_min_between(xs, 2, 5));

  vec_free(xs);
}

void test_sort(void) {
  Vec *xs = vec_new();

  vec_push(xs, 10);
  vec_push(xs, -20);
  vec_push(xs, 40);
  vec_push(xs, 10);
  vec_push(xs, -50);

  vec_sort(xs);

  TEST_ASSERT_EQUAL_INT(5, vec_length(xs));
  TEST_ASSERT_EQUAL_INT(-50, *vec_at(xs, 0));
  TEST_ASSERT_EQUAL_INT(-20, *vec_at(xs, 1));
  TEST_ASSERT_EQUAL_INT(10, *vec_at(xs, 2));
  TEST_ASSERT_EQUAL_INT(10, *vec_at(xs, 3));
  TEST_ASSERT_EQUAL_INT(40, *vec_at(xs, 4));

  vec_free(xs);
}

int main(void) {
  UNITY_BEGIN();
  RUN_TEST(test_alloc);
  RUN_TEST(test_push);
  RUN_TEST(test_pop);
  RUN_TEST(test_min_between);
  RUN_TEST(test_sort);
  return UNITY_END();
}
